<body data-spy="scroll" data-target=".topbar" data-offset="90">
<div class="topbar navbar navbar-fixed-top navbar-inverse">
  <div class="navbar-inner">
   <div class="container">
    <div class="topbar-content">

    <a class="brand" href="https://github.com/linkedin/naarad">naarad</a>

    <ul class="nav">
        {% if summary_enabled %}
        <li>
            <a href="summary_report.html" title="Summary">
                <span>Summary</span>
            </a>
        </li>
        {% endif %}
        <li class="dropdown">
            <a href="#" class="dropdown-toggle" data-toggle="dropdown">
                <span>Metrics</span>
                <b class="caret"></b>
            </a>
            <ul class="dropdown-menu">
                {%- for metric_name in metric_list %}
                <li>
                    <a href="{{ metric_name.label }}_report.html">
                        <span>{{ metric_name.label }}</span>
                    </a>
                </li>
                {%- endfor %}
            </ul>
        </li>
        <li class="active"><a href="report.html"><span>Chart</span></a></li>
    </ul>
    </div><!-- /topbar-content -->
   </div><!-- /container -->
 </div><!-- /topbar-inner -->
</div><!-- /topbar -->
<div id="chart-parent-div" class="container">
  <div id="chart-div-1" class="content">
    <div id="selection-div-1" class="row">
      <div class="span3">
        <label>
            <select id="select-chart-1" onchange="plot('select-chart-1','charting-div-1',0,false,'csv-url-div-1')">
                <option selected disabled hidden value=''>Select...</option>
                {%- for data_series in timeseries_data %}
                <option value="{{ resource_path }}/{{ data_series }}.csv">{{ data_series }}</option>
                {%- endfor %}
            </select>
        </label>
      </div>
      <div class="span3">
        <label>
            <select id="select-percentiles-1" onchange="plot_cdf('select-percentiles-1','charting-div-1',0,false,'csv-url-div-1')">
                <option selected disabled hidden value=''>Select...</option>
                {%- for data_series in percentiles_data %}
                <option value="{{ resource_path }}/{{ data_series }}.csv">{{ data_series }}</option>
                {%- endfor %}
            </select>
        </label>
      </div>
      <div class="span3">
        <button type="button" class="btn" onclick="add_chart('chart-parent-div')">+</button>
      </div>  
    </div>
    <div id="labels-charting-div-1"></div>
    <div id="charting-div-1" style="height:50%; width:100%;" class="row"></div>
    <div id="csv-url-div-1"></div>
 </div>
</div>
</body>
