<!DOCTYPE html>
<html>
  <head>
    {%- if report_title %}
      <title>{{ report_title }}</title>
    {%- else %}
      <title>naarad analysis report</title>
    {%- endif %}
    {%- if custom_javascript_includes %}
      {%- for custom_javascript in custom_javascript_includes %}
        {%- if 'http' in custom_javascript %}
         <script type="text/javascript" src="{{ custom_javascript }}"></script>
        {%- else %}
        <script type="text/javascript" src="{{ resource_path }}/{{ custom_javascript }}"></script>
        {% endif %}
      {%- endfor %}
    {%- endif %}
    {%- if custom_stylesheet_includes %}
      {%- for custom_stylesheet in custom_stylesheet_includes %}
        {%- if 'http' in custom_stylesheet %}
          <link rel="stylesheet" href="{{ custom_stylesheet }}">
        {%- else %}
        <link rel="stylesheet" href="{{ resource_path }}/{{ custom_stylesheet }}">
        {%- endif %}
      {%- endfor %}
    {%- endif %}
    <style>
      body { padding-top: 60px; }
      .popover { z-index: 10001; }
      /* make link targets appear just below the header */
      .link-target { padding-top: 80px; margin-top: -80px; }
    </style>
  </head>
